@@ -3,6 +3,8 @@ Render to qt from agg
 """
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
+import logging
+logging.debug('Loading backend_qt5agg.py')
 
 import sys
 import ctypes
@@ -11,13 +13,12 @@ import matplotlib
 from matplotlib.figure import Figure
 
 from matplotlib.backends.backend_agg import FigureCanvasAgg
-from PyQt5.QtCore import *
-from PyQt5.QtGui import *
-from backend_qt5 import FigureManagerQT
-from backend_qt5 import FigureCanvasQT
-from backend_qt5 import NavigationToolbar2QT
-from backend_qt5 import show
-from backend_qt5 import draw_if_interactive
+from .qt import *
+from .backend_qt5 import FigureManagerQT
+from .backend_qt5 import FigureCanvasQT
+from .backend_qt5 import NavigationToolbar2QT
+from .backend_qt5 import show
+from .backend_qt5 import draw_if_interactive
 
 DEBUG = False
 
@@ -141,7 +142,7 @@ class FigureCanvasQTAgg(FigureCanvasQT, FigureCanvasAgg):
                                   QImage.Format_ARGB32)
             pixmap = QPixmap.fromImage(qImage)
             p = QPainter(self)
-            p.drawPixmap(QPoint(l, self.renderer.height-t), pixmap)
+            p.drawPixmap(QPoint(l, self.renderer.height - t), pixmap)
             p.end()
             self.blitbox = None
         self.drawRect = False
@@ -164,8 +165,9 @@ class FigureCanvasQTAgg(FigureCanvasQT, FigureCanvasAgg):
         self.blitbox = bbox
         l, b, w, h = bbox.bounds
         t = b + h
-        self.repaint(l, self.renderer.height-t, w, h)
+        self.repaint(l, self.renderer.height - t, w, h)
 
     def print_figure(self, *args, **kwargs):
         FigureCanvasAgg.print_figure(self, *args, **kwargs)
         self.draw()
+
