@@ -1,16 +1,17 @@
 # -*- coding: utf-8 -*-
 # Experimental data manager
 # Loads a csv data file and extracts key information into usable structures for analysis
+from __future__ import unicode_literals
+import logging
+logging.debug('Loading data.py')
 
 # Import PyQt5 classes
-from PyQt5.QtGui import *
-from PyQt5.QtCore import *
-from PyQt5.QtWidgets import *
+from .qt import *
 
 import os, sys, re, base64
 import numpy as np
 
-from collections import defaultdict
+from collections import defaultdict, OrderedDict
 import operator
 
 from copy import copy, deepcopy
@@ -20,6 +21,7 @@ try:
 except ImportError:
     import xml.etree.ElementTree as et
 
+import logging
 
 
 
@@ -193,6 +195,8 @@ class DataManager( QObject ):
     consumed = pyqtSignal( tuple, tuple )
     unconsumed = pyqtSignal( tuple, tuple )
 
+    interfaces_changed = pyqtSignal()
+
     def __init__(self, parent, view, *args, **kwargs):
         super(DataManager, self).__init__( *args, **kwargs)
 
@@ -243,18 +247,16 @@ class DataManager( QObject ):
             self.o[interface].previously_managed_by.append(self)
             self.notify_watchers(interface)
 
-
             self.m.dataModel.refresh()
             return True
             
-            
         return False
     
     def unput(self, interface):
-        print 'UNPUTTING'
+        logging.debug('Unputting data on interface %s' % interface)
         # Trigger _unconsume on all watchers
         for w in list( self.watchers[interface] ):
-            for i,d in w.i.items():
+            for i,d in list(w.i.items()):
                 w.unget( i )
         
         self.watchers[interface] = set()
@@ -285,6 +287,8 @@ class DataManager( QObject ):
                 self.m.datasets.append( dso )
             except:
                 pass
+                
+        self.interfaces_changed.emit()
         
     def remove_output(self, interface):
         if interface in self.o:
@@ -292,6 +296,7 @@ class DataManager( QObject ):
             del self.o[ interface ]
             self.notify_watchers( interface )
             del self.watchers[ interface ]            
+            self.interfaces_changed.emit()
             return True
         return False
         
@@ -299,6 +304,7 @@ class DataManager( QObject ):
     def add_input(self, interface):
         if interface not in self.i:
             self.i[interface] = None
+            self.interfaces_changed.emit()
             return True
         else:
             return False
@@ -307,6 +313,7 @@ class DataManager( QObject ):
         if interface in self.i:
             self._unconsume( self.i[interface] )
             del self.i[interface]
+            self.interfaces_changed.emit()        
             return True
         return False        
         
@@ -329,7 +336,7 @@ class DataManager( QObject ):
             
         # Don't add data which belongs to a child
         if self in data.previously_managed_by:
-            print data.previously_managed_by
+            print(data.previously_managed_by)
             return False
 
         for consumer_def in consumer_defs:
@@ -354,7 +361,7 @@ class DataManager( QObject ):
     def _unconsume(self, data):
         if data and self in data.manager.watchers[ data.manager_interface ]:
             data.manager.watchers[ data.manager_interface ].remove( self )
-            for interface,d in self.i.items():
+            for interface, d in list(self.i.items()):
                 if d == data:
                     self.unconsumed.emit( (data.manager, data.manager_interface), (self, interface) )
 
@@ -433,10 +440,10 @@ class DataManager( QObject ):
 
 
     def reset(self):
-        for i in self.i.keys():
+        for i in list(self.i.keys()):
             self.unget(i)
         
-        for i in self.o.keys():
+        for i in list(self.o.keys()):
             self.unput(i)
             
         
@@ -476,7 +483,7 @@ class DataDefinition( QObject ):
             return self.cmp_map['aloeic'], s
             
         s = str(s) # Treat all input as strings
-        for k,v in self.cmp_map.items():
+        for k,v in list(self.cmp_map.items()):
             if k in s:
                 return v, s.replace(k,'')
         return self.cmp_map['='], s
@@ -491,20 +498,21 @@ class DataDefinition( QObject ):
         # if we fail at any point return False
         # self.interface holds the interface for this 
         # Test each option; if we get to the bottom we're alright!
-        print "CONSUME? [%s]" % data.name
-        print self.definition
-        for k,v in self.definition.items():
+        logging.debug("CONSUME? [%s]" % data.name)
+        logging.debug(self.definition)
+        
+        for k,v in list(self.definition.items()):
             t = getattr( data, k )
-            print " COMPARE: %s %s %s" % (k,v,t)
+            logging.debug(" COMPARE: %s %s %s" % (k,v,t))
             # t = (1d,2d,3d)
             # Dimensionality check
             if len(v) != len(t):
-                print "  dimensionality failure %s %s" %( len(v), len(t) )
+                logging.debug("  dimensionality failure %s %s" %( len(v), len(t) ))
                 return False
                 
             for n, cr in enumerate(v):
                 if cr == None: # No restriction on this definition
-                    print '  pass'
+                    logging.debug('  pass')
                     continue 
 
                 cmp_fn, crr = self.get_cmp_fn( cr )
@@ -512,15 +520,15 @@ class DataDefinition( QObject ):
                     crr = type(t[n])(crr) 
                 except:
                     # If we can't match equivalent types; it's nonsense so fail
-                    print "  type failure %s %s" %( type(t[n]), type(crr) )
+                    logging.debug("  type failure %s %s" %( type(t[n]), type(crr) ))
                     return False
 
                 "  comparison %s %s %s = %s" %( t[n], cmp_fn, crr, cmp_fn( t[n], crr))
                 if not cmp_fn( t[n], crr):
-                    print "  comparison failure %s %s %s" %( t[n], cmp_fn, crr )
+                    logging.debug("  comparison failure %s %s %s" %( t[n], cmp_fn, crr ))
                     return False                                
             
-        print " successful"
+        logging.debug(" successful")
         return True
 
 
@@ -660,6 +668,26 @@ class DataSet( QObject ):
         self.data = np.zeros( size ) #np.array([]) # Data container  
         
         self.metadata = {}
+        
+    # DESTRUCTIVE resizing of the current dso
+    # All entries are simply clipped to size
+    def crop(self,shape):
+
+        final_shape = list( self.data.shape )
+        for d, s in enumerate( shape ):
+            if s<self.data.shape[d]: # Only allow crop
+                self.labels[d] = self.labels[d][:s]
+                self.entities[d] = self.entities[d][:s]
+                self.scales[d] = self.scales[d][:s]
+                self.classes[d] = self.classes[d][:s]
+                final_shape[d] = shape[d]
+                
+        self.data.resize( final_shape )
+        
+    
+    @property 
+    def is_empty(self):
+        return self.shape == (0,)
               
     
     def import_data(self, dso):
@@ -744,7 +772,7 @@ class DataSet( QObject ):
     
     @property
     def scales_r(self):
-        return [ (min(s), max(s)) for s in self.scales if s is not None ]
+        return [ (min([n for n in s if n is not None]), max([n for n in s if n is not None])) for s in self.scales if s is not None]
             
     # Types (most useful on entities)
     @property
@@ -817,11 +845,12 @@ class DataSet( QObject ):
         identities = [ tuple(o) for o in zip( *[ dso.__dict__[ma][dim] for ma in match_attribs ] )  ]#dso.classes[dim], dso.labels[dim], dso.entities[dim]) ]
         identities_na = np.array( identities )
         
-        unique = tuple( set( identities ) )
+        # Unique values, without affecting order
+        unique = list(OrderedDict.fromkeys( identities )) #tuple( set( identities ) )
         
         old_shape, new_shape = dso.data.shape, list( dso.data.shape )
         new_shape[ dim ] = len( unique )
-        print 'Reshape from %s to %s' % (old_shape,new_shape)
+        print('Reshape from %s to %s' % (old_shape,new_shape))
         dso.crop( new_shape )
         
         # OPTIMISE?
@@ -885,7 +914,7 @@ class DataSet( QObject ):
             
         new_shape[dim] = list(mask).count(True) # New size of it
 
-        print 'Reshape from %s to %s' % (old_shape,new_shape)
+        print('Reshape from %s to %s' % (old_shape,new_shape))
         dso.crop( new_shape )
         # FIXME: Hacky; what about 3d arrays
         if dim == 0:
@@ -902,21 +931,6 @@ class DataSet( QObject ):
     
     def as_copy(self):
         return deepcopy(self)
-        
-    # DESTRUCTIVE resizing of the current dso
-    # All entries are simply clipped to size
-    def crop(self,shape):
-
-        final_shape = list( self.data.shape )
-        for d, s in enumerate( shape ):
-            if s<self.data.shape[d]: # Only allow crop
-                self.labels[d] = self.labels[d][:s]
-                self.classes[d] = self.classes[d][:s]
-                self.entities[d] = self.entities[d][:s]
-                self.scales[d] = self.scales[d][:s]
-                final_shape[d] = shape[d]
-                
-        self.data.resize( final_shape )
 
 
     # DESTRUCTIVE remove invalid data from this dataset object
@@ -946,39 +960,7 @@ class DataSet( QObject ):
             if d == 1:
                 self.data = self.data[ :,mask ]            
 
-        print self.labels
                     
-    # JSON
-    
-    #def json_dumps(self):
-    #    # Build JSONable structure for output
-    #    return json.dumps( self.config ) 
-    
-    #def json_loads(self, json):
-    #    # Reonstruct the object from the json input
-    #    self.config = json.loads( json )
-
-    #o = DataSet( size=self.shape )
-    #o.manager = None # Maintain the manager link
-    #o.manager_interface = None # Interface the manager is advertising this on
-
-    #o.name = deepcopy(self.name, memo)
-    #o.description = deepcopy(self.description, memo)
-    #o.type = deepcopy(self.type, memo)
-
-    #o.labels = deepcopy(self.labels, memo)
-    #o.entities = [copy(x) for x in self.entities] # deepcopy(self.entities, memo) ; this is full of pointers to database objects
-    #o.scales = deepcopy(self.scales, memo)
-    #o.classes = deepcopy(self.classes, memo)
-
-    #o.data = deepcopy(self.data)
-
-    #o.log = deepcopy(self.log)
-
-    #o.previously_managed_by = [n for n in self.previously_managed_by]
-
-    #return o   
-    
     def annotations_to_XML(self, et, obj, anno):
         # Labels (all axes)
         objset = et.SubElement(root, anno[0])
