@@ -1,40 +1,52 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-
 from __future__ import unicode_literals
+import logging
+logging.debug('Loading views.py')
+
 
 # Import PyQt5 classes
-from PyQt5.QtGui import *
-from PyQt5.QtCore import *
-from PyQt5.QtWebKit import *
-from PyQt5.QtWidgets import *
-from PyQt5.QtWebKitWidgets import *
-from PyQt5.QtPrintSupport import *
+from .qt import *
 
 from collections import defaultdict
 
-import os, urllib, urllib2, copy, re, json, importlib, sys, traceback
+import os, copy, re, json, importlib, sys, traceback, logging
+
+try:
+    from urllib.request import urlopen
+    from urllib.parse import urlparse
+except ImportError:
+    from urlparse import urlparse
+    from urllib import urlopen
+
+from collections import OrderedDict
 
 # Pathomx classes
-import utils, threads
+from . import utils, threads
 
 import numpy as np
 
-import data, config
+from . import data, config, utils
+
+from .styles import styles
 
 
 # Translation (@default context)
-from translate import tr
+from .translate import tr
 
 
 from numpy import arange, sin, pi
-from backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
-#from backend_qt5agg import NavigationToolbar2QTAgg
+from .backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
 from matplotlib.backend_bases import NavigationToolbar2
 from matplotlib.figure import Figure
 from matplotlib.colors import Colormap
+from matplotlib.path import Path
+from matplotlib.patches import BoxStyle
+
 import matplotlib.cm as cm
 
+#import seaborn
+
 progname = os.path.basename(sys.argv[0])
 progversion = "0.1"
 
@@ -60,16 +72,16 @@ BLANK_DEFAULT_HTML = '''
 # Handler for the views available for each app. Extended implementation of the QTabWidget
 # to provide extra features, e.g. refresh handling, auto focus-un-focus, status color-hinting
 class ViewManager( QTabWidget ):
-    ''' 
+    """ 
     Manager class for the tool views.
     
     Inherits from QTabWidget to focusing tabs on add and unfocus-on-refresh. The QTabWidget method
     is overridden to wrap addView. All other QTabWidget methods and attributes are available.
-    
-    '''
+    """
     auto_unfocus_tabs = ['?']
     # Signals
     source_data_updated = pyqtSignal()
+    style_updated = pyqtSignal()
     updated = pyqtSignal()
 
     def __init__(self, parent, **kwargs):
@@ -88,6 +100,7 @@ class ViewManager( QTabWidget ):
         self.data = dict() # Stores data from which figures are rendered
     
         self.source_data_updated.connect(self.onRefreshAll)
+        self.style_updated.connect(self.onRefreshAll)
     
     # A few wrappers to 
     def addView(self, widget, name, focused=True, unfocus_on_refresh=False, **kwargs):
@@ -146,7 +159,7 @@ class ViewManager( QTabWidget ):
                         self._unfocus_tabs_enabled = False # Don't do this again (so user can select whatever they want)
                         break
          
-class BaseView():
+class BaseView(object):
     """
     Base View prototype with stubs and data-handling functions that are generically useful.
     Sub-class from this if you want to create a new type of view, e.g. supporting an alternative
@@ -189,7 +202,7 @@ class BaseView():
             
             last_v = v
         
-        print "%s reduced to %s" % ( no, len(accumulator) ) 
+        print("%s reduced to %s" % ( no, len(accumulator) )) 
         return accumulator
         
         
@@ -245,16 +258,16 @@ class RenderPageToFile(QWebPage):
         self.finished = True
 
 
-class QWebPageJSLog(QWebPage):
+class WebPageJSLog(QWebPage):
     """
     Redirects Javascript errors to the console (STDOUT) for debugging.
     """
 
     def __init__(self, parent=None, **kwargs):
-        super(QWebPageJSLog, self).__init__(parent, **kwargs)
+        super(WebPageJSLog, self).__init__(parent, **kwargs)
 
     def javaScriptConsoleMessage(self, msg, lineNumber, sourceID):
-        print "JsConsole(%s:%d): %s" % (sourceID, lineNumber, msg)
+        logging.debug("JsConsole(%s:%d): %s" % (sourceID, lineNumber, msg))
 
 class QWebPageExtend(QWebPage):
     def shouldInterruptJavascript():
@@ -280,7 +293,7 @@ class WebView(QWebView, BaseView):
         
         self.w = parent
         self.m = parent.m
-        self.setPage( QWebPageJSLog(self.w) )
+        self.setPage( WebPageJSLog(self.w) )
         self.setHtml(BLANK_DEFAULT_HTML,QUrl("~"))
 
         self.page().setContentEditable(False)
@@ -308,6 +321,15 @@ class WebView(QWebView, BaseView):
         self.page().currentFrame().addToJavaScriptWindowObject("QtWebView", self)
         self.page().currentFrame().evaluateJavaScript( "QtViewportSize={'x':%s,'y':%s};" % ( sizer.width()-30, sizer.height()-80 ) ) #-magic number for scrollbars (ugh)        
         self.page().currentFrame().evaluateJavaScript( "_pathomx_render_trigger();" )
+
+ 
+    def saveAsImage(self,settings): # Size, dots per metre (for print), resample (redraw) image
+        filename, _ = QFileDialog.getSaveFileName(self, 'Save current figure', '',  "Tagged Image File Format (*.tif);;\
+                                                                                     Portable Network Graphics (*.png)")
+        if filename:
+            r = RenderPageToFile(self, filename, settings ) 
+            while r.finished != True:
+                QCoreApplication.processEvents()
     
     def getSize(self):
         if self.w.size() == QSize(100,30):
@@ -340,14 +362,7 @@ class D3View(WebView):
 
     def setSVG(self, svg):
         super(D3View, self).setHtml(svg, QUrl('file:///') )             
- 
-    def saveAsImage(self,settings): # Size, dots per metre (for print), resample (redraw) image
-        filename, _ = QFileDialog.getSaveFileName(self, 'Save current figure', '',  "Tagged Image File Format (*.tif);;\
-                                                                                     Portable Network Graphics (*.png)")
-        if filename:
-            r = RenderPageToFile(self, filename, settings ) 
-            while r.finished != True:
-                QCoreApplication.processEvents()
+
 
     def generate_d3(self, metadata):
         metadata['htmlbase'] = os.path.join( utils.scriptdir,'html')
@@ -439,7 +454,7 @@ class MplView(FigureCanvas, BaseView):
         # Install navigation handler; we need to provide a Qt interface that can handle multiple 
         # plots in a window under separate tabs
         self.navigation = MplNavigationHandler( self )
-        self.navigation.zoom()
+        
 
     def generate(self):
         pass
@@ -456,6 +471,7 @@ class MplView(FigureCanvas, BaseView):
             dpi = settings.get_dots_per_inch()
             prev_size = self.fig.get_size_inches()
             self.fig.set_size_inches(*size)
+            
             self.fig.savefig(filename, dpi=dpi)
             self.fig.set_size_inches(*prev_size)
             self.redraw()
@@ -464,40 +480,39 @@ class MplView(FigureCanvas, BaseView):
         #FIXME: Ugly hack to refresh the canvas
         self.resize( self.size() - QSize(1,1) )
         self.resize( self.size() + QSize(1,1) )
-            
+        
+    def resizeEvent(self,e):
+        FigureCanvas.resizeEvent(self,e)
+        
+
+    def get_text_bbox_screen_coords(self, t):
+        bbox = t.get_window_extent(self.get_renderer())        
+        return bbox.get_points()
+
+    def get_text_bbox_data_coords(self, t):
+        bbox = t.get_window_extent(self.get_renderer())        
+        axbox = bbox.transformed(self.ax.transData.inverted())
+        return axbox.get_points()
+        
+    def extend_limits(self, a, b):
+        # Extend a to meet b where applicable
+        ax, ay = list(a[0]), list(a[1])
+        bx, by = b[:,0], b[:,1]
+   
+        ax[0] = bx[0] if bx[0] < ax[0] else ax[0]
+        ax[1] = bx[1] if bx[1] > ax[1] else ax[1]
+
+        ay[0] = by[0] if by[0] < ay[0] else ay[0]
+        ay[1] = by[1] if by[1] > ay[1] else ay[1]
+                
+        return [ax,ay]
     
 class D3PrerenderedView(D3View):
     
     def generate(self, metadata, template):
         self.d3_template = template
         self.generate_d3( metadata )
-            
-                
-class D3HomeView(D3View):
-
-    d3_template = 'd3/workspace.svg'
-
-    def generate(self):
-        objects = []
-        for a in self.m.apps:
-            objects.append( (a.id, a) )
-
-        inheritance = [] # Inter datasetmanager links; used for view hierarchy
-        for a in self.m.apps:
-            # v.id = origin
-            for i,ws in a.data.watchers.items():
-                for w in ws:
-                    inheritance.append( (a.id, w.v.id) ) # watcher->app>id
-                    
-        links = [] # Links inputs -> outputs (dso links)
-        
-
-        template = self.m.templateEngine.get_template(self.d3_template)
-        self.setSVG(template.render( {'htmlbase': os.path.join( utils.scriptdir,'html'), 'objects':objects, 'inheritance':inheritance} )) 
-
-
-
-
+    
 
 class D3ForceView(D3View):
 
@@ -528,7 +543,6 @@ class D3ForceView(D3View):
         # In template:
         # Loop metabolites (nodes; plus their groups)
     
-    
         metadata = { 'htmlbase': os.path.join( utils.scriptdir,'html'),
                      'pathways':[self.parent.db.pathways[p] for p in self.parent.config.value('/Pathways/Show').split(',')],
                      'metabolites':metabolites,
@@ -551,10 +565,10 @@ class D3SpectraView(D3View):
             
         # If we have scale data, enable and render the Viewer tab
         #FIXME: Must be along the top axis 
-        print "Scale data up top; make a spectra view"
+        print("Scale data up top; make a spectra view")
         metadata = {'htmlbase': os.path.join( utils.scriptdir,'html')}
     
-        dso_z = zip( dso.scales[1], dso.entities[1], dso.labels[1] )
+        dso_z = list(zip( dso.scales[1], dso.entities[1], dso.labels[1] ))
 
         # Compress data along the 0-dimensions by class (reduce the number of data series; too large)                
         dsot = dso.as_summary(dim=0, match_attribs=['classes'])
@@ -569,7 +583,7 @@ class D3SpectraView(D3View):
         data = data[:,sp]
         
         metadata['figure'] = {
-            'data':zip( scale, data.T ), # (ppm, [data,data,data])
+            'data':list(zip( scale, data.T )), # (ppm, [data,data,data])
             'compounds': self.build_markers( dso_z, 1, self._build_entity_cmp ),
             'labels': self.build_markers( dso_z, 2, self._build_label_cmp ),
         }
@@ -577,6 +591,65 @@ class D3SpectraView(D3View):
         template = self.m.templateEngine.get_template(self.d3_template)
         self.setSVG(template.render( metadata ))
         
+        
+class EntityBoxStyle(BoxStyle._Base):
+    """
+    A simple box.
+    """
+
+    def __init__(self, pad=0.1):
+        """
+        The arguments need to be floating numbers and need to have
+        default values.
+
+         *pad*
+            amount of padding
+        """
+
+        self.pad = pad
+        super(EntityBoxStyle, self).__init__()
+
+    def transmute(self, x0, y0, width, height, mutation_size):
+        """
+        Given the location and size of the box, return the path of
+        the box around it.
+
+         - *x0*, *y0*, *width*, *height* : location and size of the box
+         - *mutation_size* : a reference scale for the mutation.
+
+        Often, the *mutation_size* is the font size of the text.
+        You don't need to worry about the rotation as it is
+        automatically taken care of.
+        """
+
+        # padding
+        pad = mutation_size * self.pad
+
+        # width and height with padding added.
+        width, height = width + 2.*pad, \
+                        height + 2.*pad,
+
+        # boundary of the padded box
+        x0, y0 = x0-pad, y0-pad,
+        x1, y1 = x0+width, y0 + height
+
+        cp = [(x0, y0),
+              (x1, y0), (x1, y1), (x0, y1),
+              (x0-pad, (y0+y1)/2.), (x0, y0),
+              (x0, y0)]
+
+        com = [Path.MOVETO,
+               Path.LINETO, Path.LINETO, Path.LINETO,
+               Path.LINETO, Path.LINETO,
+               Path.CLOSEPOLY]
+
+        path = Path(cp, com)
+
+        return path
+
+
+# register the custom style
+BoxStyle._style_list["entity-tip"] = EntityBoxStyle
 
 class MplSpectraView(MplView):
     """
@@ -588,52 +661,100 @@ class MplSpectraView(MplView):
     def __init__(self, parent, **kwargs):
         super(MplSpectraView, self).__init__(parent, **kwargs)        
         self.ax.invert_xaxis()
+        
 
     def generate(self, dso=None):
+    
         if not float in [type(t) for t in dso.scales[1]]:   
             # Add fake axis scale for plotting
-            dso.scales[1] = range( len(dso.scales[1]) )
+            dso.scales[1] = list(range( len(dso.scales[1])))
             
-        #FIXME: Should probably be somewhere else. Label up the top 50    
+        #FIXME: Should probably be somewhere else. Label up the top N
         wmx = np.amax( np.absolute( dso.data), axis=0 )
-        dso_z = zip( dso.scales[1], dso.entities[1], dso.labels[1] )
-        dso_z = sorted( zip( dso_z, wmx ), key=lambda x: x[1])[-20:] # Top 50
-        dso_z = [x for x, wmx in dso_z ]    
+        dummy_scale = list(range(0,len(dso.scales[1]) ) )
+        dso_z = list(zip( dummy_scale, dso.entities[1], dso.labels[1] ))
+        dso_z = sorted( zip( dso_z, wmx ), key=lambda x: x[1])[-10:] # Top N
+        dso_z = [x for x, wmx in dso_z ]  
 
-        print dso_z
-        
         # Compress data along the 0-dimensions by class (reduce the number of data series; too large)                
         dsot = dso.as_summary(dim=0, match_attribs=['classes'])
 
         # Copy to sort without affecting original
         scale = np.array(dso.scales[1])
         data = np.array(dsot.data)
-        
+
         # Sort along x axis
         sp = np.argsort( scale )
         scale = scale[sp]
         data = data[:,sp]
 
-
         self.ax.cla()
 
-        self.ax.plot(scale, data.T, linewidth=0.75)
+        if dsot.shape[0] > 1: 
+            # More than one data row (class) so plot each class
+            
+            plots = OrderedDict()
+            for n,row in enumerate(data):
+                c = dsot.classes[0][n]
+                ls = styles.get_style_for_class( c )
+                plots[ c ], = self.ax.plot(scale, row, **ls.line_kwargs)
+        
+            legend = self.ax.legend(list(plots.values()),
+               list(plots.keys()),
+               loc='best') #, bbox_to_anchor=(1, 1))
+            legend.get_frame().set_facecolor('k')                      
+            legend.get_frame().set_alpha(0.05)     
+
+        else:
+            # Only one data row (class) so plot individual data; with a mean line
+            data_mean = data
+            data_individual = dso.data[:,sp]            
+            
+            for n,row in enumerate(data_individual):
+                self.ax.plot(scale, row, linewidth=0.75, alpha=0.25, color=utils.category10[0])
+            
+            self.ax.plot(scale, data_mean.T, linewidth=0.75, color=utils.category10[0])
+            
+
+        ylims = np.max(data, axis=0)
+        
+        axlimits= ( self.ax.get_xlim(), self.ax.get_ylim() )
 
         for c in self.build_markers( dso_z, 1, self._build_entity_cmp ):
-            self.ax.axvspan( c[0], c[1], facecolor='#dddddd', edgecolor='#dddddd', linewidth=0.5)
-            self.ax.text( (c[0]+c[1])/2, self.ax.get_ylim()[1], c[2], rotation='60', rotation_mode='anchor', color='#dddddd', size=6.5)
+            x = (dso.scales[1][ c[0] ] + dso.scales[1][ c[1] ]) / 2.0
+            y = max( ylims[ c[0] ], ylims[ c[1] ] )
+            if y>=0:
+                r = '60'
+            else:
+                r = '-60'
+                
+            
+            t = self.ax.text(x, y, c[2], rotation=r, rotation_mode='anchor', size=6.5, bbox=dict(boxstyle="entity-tip,pad=0.2", alpha=0.1) )
+            axlimits = self.extend_limits( axlimits, self.get_text_bbox_data_coords(t) )
 
-        #for c in self.build_markers( dso_z, 2, self._build_label_cmp ):
-        #    self.ax.axvspan( c[0], c[1], facecolor='#eeeeee', edgecolor='#eeeeee', linewidth=0.5)
-        #    self.ax.text( (c[0]+c[1])/2, self.ax.get_ylim()[1], c[2], rotation='60', rotation_mode='anchor', color='#eeeeee', size=6.5)
+        for c in self.build_markers( dso_z, 2, self._build_label_cmp ):
+            x = (dso.scales[1][ c[0] ] + dso.scales[1][ c[1] ]) / 2.0
+            y = max( ylims[ c[0] ], ylims[ c[1] ] )
+            if y>=0:
+                r = '60'
+            else:
+                r = '-60'
+
+            t = self.ax.text(x, y, c[2], rotation=r, rotation_mode='anchor', size=6.5, bbox=dict(boxstyle="entity-tip,pad=0.2", fc="#eeeeee", ec="#dddddd", alpha=0.1) ) 
+            axlimits = self.extend_limits( axlimits, self.get_text_bbox_data_coords(t) )
+
+        #self.ax.set_xlim(axlimits[0].reverse())
+        self.ax.set_ylim(axlimits[1])
 
         self.ax.set_xlabel('ppm')
         self.ax.set_ylabel('Rel')
-        print 'OK'
+
+
         self.draw()
                 
         
         
+        
 class D3DifferenceView(D3View):
     """
     A matplotlib-based difference-lineplot viewer.
@@ -655,7 +776,7 @@ class D3DifferenceView(D3View):
             datao = np.interp( dso_a.scales[1], dso_b.scales[1], datao)
 
         metadata['figure'] = {
-            'data':zip( dso_a.scales[1], datai.T, datao.T ), # (ppm, [dataa,datab])
+            'data':list(zip( dso_a.scales[1], datai.T, datao.T )), # (ppm, [dataa,datab])
         }
         
         template = self.m.templateEngine.get_template(self.d3_template)
@@ -680,8 +801,8 @@ class MplDifferenceView(MplSpectraView):
 
         self.ax.cla()
         self.ax.plot(x, y2, color='black', linewidth=0.25)
-        self.ax.fill_between(x, y1, y2, where=y2>=y1, facecolor=utils.category10[0], interpolate=True)
-        self.ax.fill_between(x, y1, y2, where=y2<=y1, facecolor=utils.category10[1], interpolate=True)
+        self.ax.fill_between(x, y1, y2, where=y2>=y1, facecolor=utils.category10[0], interpolate=False)
+        self.ax.fill_between(x, y1, y2, where=y2<=y1, facecolor=utils.category10[1], interpolate=False)
         
         self.ax.set_xlabel('ppm')
         self.ax.set_ylabel('Rel')
@@ -713,8 +834,15 @@ class MplScatterView(MplView):
         super(MplScatterView, self).__init__(parent, **kwargs)        
 
     def generate(self, dso=None): #figure_data
+        """
+        Generate an XY scatter plot from source dataset object.
+
+        :param dso: The dataset object to plot consisting of two columns (X, Y) of data.
+                    Multiple classes of data can be supplied as indicated by the axis 0 class list.
+        :type dso: DataSet object
+        """
         self.ax.cla()
-        sp = {}
+        plots = {}
         colors = self.ax._get_lines.color_cycle
 
         # Scores dso is no_of_samples x no_of_series (axes) ; only plot first two
@@ -723,12 +851,16 @@ class MplScatterView(MplView):
         classes = dso.classes_l[0]
         for c in classes:
             df = dso.as_filtered(dim=0,classes=[c])
-            sp[c] = self.ax.scatter(df.data[:,0], df.data[:,1], c=next(colors) )
+            ls = styles.get_style_for_class( c )
+            s = ls.markersize**2 if ls.markersize != None else 20 #default
+            plots[c] = self.ax.scatter(df.data[:,0], df.data[:,1], color=ls.markerfacecolor, marker=ls.marker, s=s)
 
-        self.ax.legend(sp.values(),
-           sp.keys(),
+        legend = self.ax.legend(list(plots.values()),
+           list(plots.keys()),
            scatterpoints=1,
            loc='upper left', bbox_to_anchor=(1, 1))
+        legend.get_frame().set_facecolor('k')                      
+        legend.get_frame().set_alpha(0.05)        
            
         self.ax.set_xlabel(dso.labels[1][0])
         self.ax.set_ylabel(dso.labels[1][1])
@@ -797,7 +929,7 @@ class D3CircosView(D3View):
     d3_template = 'd3/circos.svg'
 
     def generate(self, dso=None):
-        print dso
+        print(dso)
 
         self.generate_d3( {
             'figure': {
@@ -814,17 +946,17 @@ class D3CircosView(D3View):
 class AnalysisCircosPathwayView(D3LegacyView):
 
     def generate(self):
-        pathways = self.parent.db.pathways.keys()
+        pathways = list(self.parent.db.pathways.keys())
         pathway_metabolites = dict()
         
-        for k,p in self.parent.db.pathways.items():
+        for k,p in list(self.parent.db.pathways.items()):
             pathway_metabolites[p.id] = set( [m for m in p.metabolites] )
 
         data_m, labels_m = self.build_matrix(pathways, pathway_metabolites)
 
         pathway_reactions = dict()
         
-        for k,p in self.parent.db.pathways.items():
+        for k,p in list(self.parent.db.pathways.items()):
             pathway_reactions[p.id] = set( [m for m in p.reactions] )
 
         data_r, labels_r = self.build_matrix(pathways, pathway_reactions)
@@ -860,7 +992,7 @@ class D3BarView(D3View):
     def generate(self, dso): 
         fd = np.mean( dso.data, axis=0 )
 
-        fdm = zip( dso.labels[1], fd )
+        fdm = list(zip( dso.labels[1], fd ))
         sms = sorted(fdm,key=lambda x: abs(x[1]), reverse=True )
         metabolites = [m for m,s in sms]
 
@@ -902,11 +1034,11 @@ class MplCategoryBarView(MplView):
 
         # FIXME: Remove this once UI allows selection of data to plot
         fd = np.mean( dso.data, axis=0 )
-        fdm = zip( dso.labels[1], fd )
+        fdm = list(zip( dso.labels[1], fd ))
         sms = sorted(fdm,key=lambda x: abs(x[1]), reverse=True )
         labels = [m for m,s in sms]
         
-        sp = {}
+        plots = OrderedDict()
         classes = dso.classes[0]
         #labels = [e if e != None else dso.labels[1][n] for n,e in enumerate(dso.entities[1][0:limit_to]) ]
         #data = dso.data[:,0:limit_to]
@@ -942,20 +1074,21 @@ class MplCategoryBarView(MplView):
                 yerr = (ynerr, yperr)
             else:
                 yerr = None
-                
-            print yerr
 
             color = next(colors)
-            sp[c] = self.ax.bar(x[:,n], cdata, align='center', color=color, yerr=yerr, ecolor=color )
+            ls = styles.get_style_for_class( c )
+            plots[c] = self.ax.bar(x[:,n], cdata, align='center', yerr=yerr, **ls.bar_kwargs)
 
         xticks = np.mean(x,axis=1)
         self.ax.set_xticks( xticks )
         self.ax.set_xticklabels(labels, rotation=45, ha='right', rotation_mode='anchor' )
 
-        self.ax.legend(sp.values(),
-           sp.keys(),
-           scatterpoints=1,
-           loc='upper left', bbox_to_anchor=(1, 1))
+
+        legend = self.ax.legend(list(plots.values()),
+           list(plots.keys()),
+           loc='best') #, bbox_to_anchor=(1, 1))
+        legend.get_frame().set_facecolor('k')                      
+        legend.get_frame().set_alpha(0.05)     
            
         #if options.title:
         #    self.ax.title(options.title)
@@ -968,7 +1101,7 @@ class MplCategoryBarView(MplView):
         # Add some padding either side of graphs
         #plt.xlim( ind[0]-1, ind[-1]+1)
     
-        self.fig.tight_layout()
+        
         self.draw()
         
 class MplHeatmapView(MplView):
@@ -998,11 +1131,11 @@ class MplHeatmapView(MplView):
             
         self.ax.imshow( dso.data, interpolation='none', aspect='auto', vmin=-ylim, vmax=+ylim, cmap=cm.bwr)
 
-        self.ax.set_xticks( range( dso.shape[1] ) )
-        self.ax.set_yticks( range( dso.shape[0] ) )
+        self.ax.set_xticks( list(range( dso.shape[1])) )
+        self.ax.set_yticks( list(range( dso.shape[0])) )
         self.ax.set_yticklabels( dso.labels[0], size=7 )
         self.ax.set_xticklabels( dso.labels[1], rotation=60, rotation_mode='anchor', ha="left", size=7 )
     
-        self.fig.tight_layout()
+        
         self.draw()
         
